import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FindOperator, Like, Repository } from 'typeorm';
import { Carrito } from './entity/carrito.entity';
import { InstitucionTipoCarrito } from '../institucion-tipo-carrito/entity/institucion-tipo-carrito.entity';
import { Modulo } from '../modulo/entity/modulo.entity';
import { InstitucionTipoCarritoService } from '../institucion-tipo-carrito/institucion-tipo-carrito.service';
import { ModuloService } from '../modulo/modulo.service';
import { TipoCarrito } from 'src/institucion-tipo-carrito/entity/tipo-carrito.entity';

@Injectable()
export class CarritoService {
  constructor(
    @InjectRepository(Carrito) private repository: Repository<Carrito>,
    private institucionTipoCarritoService: InstitucionTipoCarritoService,
    private moduloService: ModuloService,
  ) {}

  async create(id_tipo_carrito: number, id_modulo: number) {
    const modulo = await this.moduloService.findById(id_modulo);
    const tipoCarrito =
      await this.institucionTipoCarritoService.findByIdTipoCarito(
        id_tipo_carrito,
      );
    let carrito = '';

    return this.repository
      .count({ modulo, tipoCarrito })
      .then((carritos) => {
        carrito = `C${carritos > 9 ? '' : '0'}${carritos + 1}`;
        return this.repository.findOne({ modulo, tipoCarrito, carrito });
      })
      .then((existeCarrito) => {
        if (existeCarrito)
          throw new ConflictException('Este carrito ya existe en este módulo.');
        return this.repository.save(
          this.repository.create({
            tipoCarrito,
            modulo,
            carrito,
          }),
        );
      })
      .then((_) => ({
        message: 'Se creó correctamente el carrito en el módulo.',
      }));
  }

  findAll() {
    return this.repository.find();
  }

  findById(id_carrito: number) {
    return this.repository.findOne({ id_carrito }).then((carrito) => {
      if (!carrito) throw new NotFoundException('No existe este carrito.');
      return carrito;
    });
  }

  async findByIdModulo(filtros: {
    pagina: string;
    id_modulo?: string;
    id_tipo_carrito?: string;
  }) {
    const busqueda: {
      where: {
        modulo?: Modulo;
        tipoCarrito?: TipoCarrito;
      };
      skip: number;
      take: number;
    } = {
      where: {},
      skip: (parseInt(filtros.pagina) - 1) * 25,
      take: 25,
    };
    const modulo = filtros.id_modulo
      ? await this.moduloService.findById(parseInt(filtros.id_modulo))
      : null;
    const tipoCarrito = filtros.id_tipo_carrito
      ? await this.institucionTipoCarritoService.findTipoCarritoById(
          parseInt(filtros.id_tipo_carrito),
        )
      : null;

    if (modulo) busqueda.where.modulo = modulo;
    if (tipoCarrito) busqueda.where.tipoCarrito = tipoCarrito;

    return this.repository.find(busqueda);
  }

  async existeCarrito(id_modulo: number | Modulo, carrito: string) {
    const modulo =
      typeof id_modulo === 'number'
        ? await this.moduloService.findById(id_modulo)
        : id_modulo;

    return this.repository
      .findOne({ modulo, carrito })
      .then((existeCarrito) => {
        if (existeCarrito)
          throw new ConflictException(
            'Ya existe un carrito con este nombre, intenta con otro.',
          );
      });
  }

  async update(attrs: Partial<Carrito>) {
    return this.findById(attrs.id_carrito)
      .then(async (carrito) => {
        if (attrs.carrito)
          await this.existeCarrito(carrito.modulo, attrs.carrito);
        Object.assign(carrito, attrs);
        return this.repository.save(carrito);
      })
      .then((_) => ({
        message: 'Se actualizo correctamente la información del carrito.',
      }));
  }
}
